package com.anjun.core.tool

import androidx.activity.result.contract.ActivityResultContracts
import androidx.core.app.ActivityCompat
import androidx.fragment.app.Fragment
import androidx.fragment.app.FragmentActivity
import androidx.fragment.app.commit
import androidx.lifecycle.LifecycleOwner
import com.anjun.core.tool.ext.launchWhenResumed
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.withContext
import java.lang.RuntimeException

/**
 *
 * @Description
 * @Author melle
 * @CreateTime 2024/03/12 09:42:36
 */

typealias PermissionCallBack = (granted: Boolean, deniedList: Array<String>, alwaysDeniedList: Array<String>) -> Unit

fun LifecycleOwner.requestPermission(
    vararg permissions: String,
    block: PermissionCallBack
) {
    requestPermission(block,permissions.toList().toTypedArray())
}

fun LifecycleOwner.requestPermission( block: PermissionCallBack,permissions: Array<String>) {
    runCatching {
        when (this@requestPermission) {
            is FragmentActivity -> {
                PermissionFragment.getPermissionFragment(this@requestPermission, block)
                    .requestPermission(permissions)
            }

            is Fragment -> {
                PermissionFragment.getPermissionFragment(requireActivity(), block)
                    .requestPermission(permissions)
            }

            else -> throw RuntimeException("requestPermission LifecycleOwner必须是activity或fragment")
        }
    }.onFailure {
        block(false, permissions, arrayOf())
    }
}

/**
 * 权限申请代理fragment
 * 由于launcher不能在activity使用时随时创建，因此需要代理fragment，以便于回调能随时接收、修改
 */
class PermissionFragment : Fragment() {

    companion object {
        private const val TAG = "PermissionFragment"

        fun getPermissionFragment(
            activity: FragmentActivity,
            block: PermissionCallBack
        ): PermissionFragment {
            return (findPermissionFragment(activity) ?: PermissionFragment().also { fragment ->
                activity.supportFragmentManager.commit(true) {
                    add(fragment, TAG)
                }
            }).also {
                it.block = block
            }
        }

        private fun findPermissionFragment(activity: FragmentActivity): PermissionFragment? {
            return activity.supportFragmentManager.findFragmentByTag(TAG) as? PermissionFragment
        }
    }

    private var block: PermissionCallBack = { _, _, _ -> }

    fun requestPermission(permissions: Array<String>) {
        launchWhenResumed {
            launcher.launch(permissions)
        }
    }

    private val launcher =
        registerForActivityResult(ActivityResultContracts.RequestMultiplePermissions()) { result ->
            launchWhenResumed {
                dealResult(requireActivity(), result, block)
                requireActivity().supportFragmentManager.commit {
                    remove(this@PermissionFragment)
                }
            }
        }

    private suspend fun dealResult(
        activity: FragmentActivity,
        result: Map<String, Boolean>,
        block: PermissionCallBack
    ) = withContext(Dispatchers.IO) {
        val allDeniedList = result.filterValues { !it }.mapNotNull { it.key }
        val alwaysDeniedList = allDeniedList.filter {
            ActivityCompat.shouldShowRequestPermissionRationale(
                activity,
                it
            )
        }
        withContext(Dispatchers.Main) {
            block(
                allDeniedList.isEmpty(),
                (allDeniedList - alwaysDeniedList.toSet()).toTypedArray(),
                alwaysDeniedList.toTypedArray()
            )
        }
    }
}
