package com.anjun.common.base.activity

import android.Manifest
import android.annotation.SuppressLint
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.device.ScanManager
import android.os.Handler
import android.os.Looper
import android.os.Message
import androidx.activity.result.contract.ActivityResultContracts
import androidx.appcompat.app.AlertDialog
import androidx.databinding.ViewDataBinding
import com.anjun.common.base.R
import com.anjun.common.base.ext.writeToLogFile
import com.anjun.common.view.activity.ScanCodeActivity
import com.anjun.common.view.ext.parseScanResult
import com.anjun.core.tool.ext.showToast
import com.anjun.core.tool.requestPermission

private const val TAG_GET_SCAN_NUM: Int = 1
abstract class BaseScanActivity<B : ViewDataBinding> : BaseTitleActivity<B>() {

    private val mScanManager: ScanManager by lazy {
        ScanManager()
    }

    private val mHandler = Handler(Looper.getMainLooper()) {
        when (it.what) {
            TAG_GET_SCAN_NUM -> {
                onScanResult(it.obj.toString())
            }
        }
        return@Handler false
    }

    private val mReceiver = object : BroadcastReceiver() {
        override fun onReceive(context: Context?, intent: Intent?) {
            intent ?: return
            when (intent.action) {
                ScanManager.ACTION_DECODE -> {
                    intent.getStringExtra(ScanManager.BARCODE_STRING_TAG)?.let {
                        mHandler.sendMessage(Message().apply {
                            what = TAG_GET_SCAN_NUM
                            obj = it
                        })
                    }
                }
            }
        }
    }

    @SuppressLint("UnspecifiedRegisterReceiverFlag")
    override fun setupData() {
        super.setupData()
        initScan()
        //注册广播
        registerReceiver(mReceiver, IntentFilter().apply {
            addAction(ScanManager.ACTION_DECODE)
        })
    }

    override fun onDestroy() {
        super.onDestroy()
        mHandler.removeCallbacksAndMessages(null)
        try {
            unregisterReceiver(mReceiver)
        } catch (e: Exception) {
            e.printStackTrace()
            "解绑扫描receiver失败！".writeToLogFile()
        }
    }

    private fun initScan() {
        try {
            if (!mScanManager.scannerState && !mScanManager.openScanner()) {
                AlertDialog.Builder(this).setMessage(getString(R.string.s_may_cant_scan))
                    .setPositiveButton(getString(R.string.s_ok)) { dialog, _ -> dialog.dismiss() }
                    .create().show()
            } else {
                // 设置扫描输出模式： 分为了文本框聚焦输出模式(不用编码)，和广播隐式Intent模式，但这个要注册相关广播监听
                if (mScanManager.outputMode != 0) {
                    mScanManager.switchOutputMode(0)
                }
            }
        } catch (e: Exception) {
            e.printStackTrace()
            "BaseScanActivity:无法使用扫描！".writeToLogFile()
        }
    }

    private val scanLauncher =
        registerForActivityResult(ActivityResultContracts.StartActivityForResult()) {
            it.data?.parseScanResult()?.let(::onScanResult)
        }

    fun startScanCode() {
        requestPermission(Manifest.permission.CAMERA) { granted, _, _ ->
            if (granted) {
                // 跳转扫码
                try {
                    scanLauncher.launch(Intent(this, ScanCodeActivity::class.java))
                } catch (e: Throwable) {
                    e.printStackTrace()
                }
            } else {
                showToast(R.string.s_get_permission_error)
            }
        }
    }

    /**
     * 扫码结果回调
     */
    protected abstract fun onScanResult(result: String)
}