package com.anjun.common.base.ext

import androidx.appcompat.app.AppCompatActivity
import androidx.fragment.app.Fragment
import androidx.lifecycle.ViewModel
import androidx.lifecycle.lifecycleScope
import androidx.lifecycle.viewModelScope
import com.anjun.common.base.AJBaseResponse
import com.anjun.common.base.ResponseCode
import com.anjun.common.view.ext.dismissWaitDialog
import com.anjun.common.view.ext.showWaitDialog
import com.anjun.core.mvvm.ActivityManager
import com.anjun.core.tool.ext.showToast
import com.anjun.data.main.UserMMKV
import com.anjun.network.BaseRequest
import com.anjun.network.Repo
import com.anjun.network.RequestResult
import com.anjun.network.ext.repo
import com.anjun.network.ext.request
import com.anjun.network.onFailure
import com.anjun.service.main.MainProxy
import kotlinx.coroutines.CoroutineScope


/**
 *
 * @Description
 * @Author melle
 * @CreateTime 2024/03/06 18:09:16
 */

typealias ResultCallBack<T> = (RequestResult<AJBaseResponse<T>>) -> Unit

/**
 * 请求接口，以AJBaseResponse数据类型返回
 */
inline fun <reified T> CoroutineScope.requestData(
    repo: Repo,
    crossinline callback: ResultCallBack<T>,
) {
    request(repo, callback, getAJBaseResponseParameterizedType<T>())
}

/**
 * 请求接口，以AJBaseResponse数据类型返回
 */
inline fun <reified T1, reified T2> CoroutineScope.requestData(
    repo1: Repo,
    repo2: Repo,
    crossinline callback: (RequestResult<Pair<AJBaseResponse<T1>, AJBaseResponse<T2>>>) -> Unit
) {
    request(
        repo1,
        repo2,
        callback,
        getAJBaseResponseParameterizedType<T1>(),
        getAJBaseResponseParameterizedType<T2>()
    )
}

/**
 * 请求接口，以AJBaseResponse数据类型返回
 */
inline fun <reified T1, reified T2, reified T3> CoroutineScope.requestData(
    repo1: Repo,
    repo2: Repo,
    repo3: Repo,
    crossinline callback: (RequestResult<Triple<AJBaseResponse<T1>, AJBaseResponse<T2>, AJBaseResponse<T3>>>) -> Unit
) {
    request(
        repo1,
        repo2,
        repo3,
        callback,
        getAJBaseResponseParameterizedType<T1>(),
        getAJBaseResponseParameterizedType<T2>(),
        getAJBaseResponseParameterizedType<T3>(),
    )
}

/**
 * 请求接口，以AJBaseResponse数据类型返回
 */
inline fun <reified T> Repo.requestData(
    coroutineScope: CoroutineScope,
    crossinline callback: ResultCallBack<T>,
) {
    request(coroutineScope, callback, getAJBaseResponseParameterizedType<T>())
}

/**
 * 请求成功时的处理，code=200时请求成功。
 */
inline fun <reified V, reified T : AJBaseResponse<V>> RequestResult<T>.onSuccessCode(block: (T) -> Unit) {
    if (this is RequestResult.Success && ResponseCode.SUCCESS.code == value.code) {
        block(value)
    }
}

/**
 * 其他code的处理，code!=200时调用。
 */
inline fun <reified V, reified T : AJBaseResponse<V>> RequestResult<T>.onOtherCode(block: (T) -> Unit) {
    if (this is RequestResult.Success && ResponseCode.SUCCESS.code != value.code) {
        block(value)
    }
}

inline fun <reified T> AppCompatActivity.request(
    init: BaseRequest.() -> Unit,
    crossinline callback: ResultCallBack<T>
) = handleRequest(lifecycleScope, init, callback)

inline fun <reified T> Fragment.request(
    init: BaseRequest.() -> Unit,
    crossinline callback: ResultCallBack<T>
) = handleRequest(lifecycleScope, init, callback)

inline fun <reified T> ViewModel.request(
    init: BaseRequest.() -> Unit,
    crossinline callback: ResultCallBack<T>
) = handleRequest(viewModelScope, init, callback)

/**
 * 拦截请求结果，处理网络异常或返回不成功等情况。
 */
inline fun <reified T> handleRequest(
    coroutineScope: CoroutineScope,
    init: BaseRequest.() -> Unit,
    crossinline callback: ResultCallBack<T>
) {
    showWaitDialog()
    repo(init).requestData(coroutineScope) { result ->
        dismissWaitDialog()
        result.onOtherCode { response ->
            response.msg?.showToast()
            when (response.code) {
                ResponseCode.IDENTIFY_INVALID.code -> {
                    //重新登录
                    UserMMKV.clear()
                    ActivityManager.topActivity()?.let {
                        ActivityManager.finishAllActivity()
                        MainProxy.launchLoginActivity(it)
                    }
                }
            }
        }
        result.onFailure {
            it.second.showToast()
        }
        callback(result)
    }
}
